precision mediump float;

#ifdef TEXTURE
varying highp vec2 v_tex_coord;
uniform lowp sampler2D s_texture;
#endif
 
#ifdef TEXTURE_YUV
uniform lowp sampler2D s_textureU;
uniform lowp sampler2D s_textureV;
#endif 

#ifdef ALPHA
uniform lowp float u_alpha;
#endif

#ifdef COLOR
uniform lowp vec4 u_color;
#endif

#ifdef SATURATE
uniform lowp float u_saturation;
#endif

#ifdef VERTEX_COLOR
varying lowp vec4 v_color;
#endif

#ifdef MASK
varying highp vec2 v_mask_tex_coord;
uniform lowp sampler2D s_mask_texture;
#endif

vec4 Desaturate(vec3 color, float desaturation, float alpha)
{
	vec3 grayXfer = vec3(0.3, 0.59, 0.11);
	vec3 gray = vec3(dot(grayXfer, color));
	return vec4(mix(color, gray, desaturation), alpha);
}

void main()
{
  vec4 res_color;

#ifdef TEXTURE
  #ifdef TEXTURE_YUV
    highp float y = texture2D(s_texture, v_tex_coord).r;
    highp float u = texture2D(s_textureU, v_tex_coord).r - 0.5;
    highp float v = texture2D(s_textureV,  v_tex_coord).r - 0.5;

    //res_color = vec4(
    //  y + 1.13983 * v,                //R
    //  y - 0.39465 * u - 0.58060 * v,  //G
    //  y + 2.03211 * u,                //B
    //  1.0);

    res_color = vec4(
      y +             1.402 * v,    //R
      y - 0.344 * u - 0.714 * v,    //G
      y + 1.772 * u,                //B
      1.0); 
  #else
    res_color = texture2D(s_texture, v_tex_coord);
  #endif
#else
 #ifdef COLOR
  res_color = u_color;
 #endif
#endif

#ifdef VERTEX_COLOR
 #ifdef TEXTURE
  #ifdef ALPHA_MASK
   if(v_color.a > 0.0)
     res_color = vec4(v_color.rgb*res_color.a, res_color.a);
  #else
   res_color *= v_color;
  #endif
 #else
  res_color = v_color;
 #endif
#endif

#ifdef SATURATE
  res_color = Desaturate(res_color.rgb, 1.0 - u_saturation, res_color.a);
#endif

#ifdef ALPHA
  res_color *= u_alpha;
#endif

#ifdef COLOR
 #ifdef TEXTURE
  #ifdef ALPHA_MASK
    #ifdef VERTEX_COLOR
     res_color *= u_color;
    #else
     res_color = vec4(u_color.rgb*res_color.a, res_color.a);
    #endif
  #else
     res_color *= u_color;
  #endif
 #endif
#endif

#ifdef MASK
  vec4 mask_color = texture2D(s_mask_texture, v_mask_tex_coord); 
  res_color *= mask_color.a;
#endif

  gl_FragColor = res_color;
}

